; These are equates, they work exactly like #define in C
; We use them here for the locations of the various data in VU memory
ProjMat			.equ 0
WorldMat		.equ 4
CamPos			.equ 8
Scale			.equ 9
GIFTag			.equ 10
StartVert		.equ 12
NumVerts		.equ 9
VertexBuffer	.equ 290

; This is a file full of macros that comes with VCL
.include "vcl_sml.i"

; Tell VCL that we want to be able to use all vf, and vi registers
.init_vf_all
.init_vi_all

.syntax new
.vu

; This is the entry point, and is where execution will start on VU1
--enter
--endenter

	iaddiu		iVert, vi00, 0				; Start vertex counter
	iaddiu		iVertPtr, vi00, 0			; Point to the first vert
	iaddiu		iNumVerts, vi00, NumVerts	; Load the loop end condition
		
	loi			128.0						; Load a white colour 
	addi.xyzw	Colour, vf00, i				; to help illustrate culling
	ftoi0		Colour, Colour
	
	lq			fScales, Scale(vi00)		; A scale vector that we will add
											; to and scale the vertices by after
											; projecting them
	
	lq			fCameraPos, CamPos(vi00)	; Camera Position in world space (for backface culling)
	
	; Store ADC Set Bit for when we need to set the ADC flag
	iaddiu		iADCNoDraw, vi00, 1
	iaddiu		iADCNoDraw, iADCNoDraw, 0x7FFF
	iaddiu		iADCDraw, vi00, 0				; Load an ADC value (draw = true)
	
	; Load the  WVP transformation matrix
	lq			fTransform[0], ProjMat+0(vi00)
	lq			fTransform[1], ProjMat+1(vi00)
	lq			fTransform[2], ProjMat+2(vi00)
	lq			fTransform[3], ProjMat+3(vi00)
	
	; Load the World transformation matrix
	lq			fWorld[0], WorldMat+0(vi00)
	lq			fWorld[1], WorldMat+1(vi00)
	lq			fWorld[2], WorldMat+2(vi00)
	lq			fWorld[3], WorldMat+3(vi00)
	
loop:										; For each vertex

	lq			Vert, StartVert(iVertPtr)	; Load the vector (currently in object space
											; and floating point format)
											
	; Transform the vertex by the World matrix we passed in (this is a macro from
	; vcl_sml.i)
	MatrixMultiplyVertex Vert, fWorld, Vert

	sq			Vert, VertexBuffer(iVert)	; Store the vertex position in the temporary vertex position
											; which we can use for back face culling later


	lq			Vert, StartVert(iVertPtr)	; Load the vector (currently in object space
											; and floating point format)
	
	; Transform the vertex by the projection matrix we passed in (this is a macro from
	; vcl_sml.i)
	MatrixMultiplyVertex Vert, fTransform, Vert
    
    div         q,    vf00[w], Vert[w]		; Work out 1.0f / w
    mul.xyz     Vert, Vert, q				; Project the vertex by dividing by W
    
    mula.xyz	acc, fScales, vf00[w]		; Move fScales into the accumulator (acc = fScales * 1.0)
 	madd.xyz	Vert, Vert, fScales			; Multiply and add the scales (Vert = Vert * fScales + fScales)
	ftoi4.xyz	Vert, Vert					; Convert the vertex to 12:4 fixed point for the GS
	
	sq.xyz		Vert, StartVert(iVertPtr)	; Write the vertex back to VU memory
	;isw.w		iADCDraw, StartVert(iVertPtr)	; Set the ADC bit to 0 to draw vertex

	iaddiu		iVert, iVert, 1				; Increment the vertex counter
	iaddiu		iVertPtr, iVertPtr, 2		; Increment the vertex pointer
	ibne		iVert, iNumVerts, loop		; Branch back to "loop" label if
											; iVert and iNumVerts are not equal
											
	;---------------------------------------------------------------------------------------------------------
	;	Back Face Culling (Per Triangle)
	;---------------------------------------------------------------------------------------------------------
	lq				Vertex1,  2+VertexBuffer(vi00)					; Read our buffered vertex positions
	lq				Vertex2,  1+VertexBuffer(vi00)	
	lq				Vertex3,  0+VertexBuffer(vi00)	
	TriangleWinding Result, Vertex1, Vertex2, Vertex3, fCameraPos	; if vi01 = 1, our face needs to be culled
	ibeq			Result,	vi00, Draw
	
	; Cull the triangle here
	;isw.w			iADCNoDraw, StartVert(vi00)
	;isw.w			iADCNoDraw, StartVert+2(vi00)
	;isw.w			iADCNoDraw, StartVert+4(vi00)
	
	sq			Colour, StartVert-1(vi00)
	sq			Colour, StartVert+1(vi00)
	sq			Colour, StartVert+3(vi00)
	
	b				End_Cull
	
	Draw:
	
	isw.w		iADCDraw, StartVert+0(vi00)	; Set the ADC bit to 0 to draw vertex
	isw.w		iADCDraw, StartVert+2(vi00)	; Set the ADC bit to 0 to draw vertex
	isw.w		iADCDraw, StartVert+4(vi00)	; Set the ADC bit to 0 to draw vertex
	

	
	End_Cull:
	;---------------------------------------------------------------------------------------------------------

											
											
	iaddiu		iGIFTag, vi00, GIFTag		; Load the position of the giftag
	xgkick		iGIFTag						; and tell the PS2 to send that to the GS
	
--exit
--endexit
